import os
import sys

import toml

from myprss import logger


CONFIG_FILE_PATH = "~/.myprss"


class ConfigMeta(type):
    """Metaclass that makes our Config a singleton"""

    _instance = None

    def __call__(cls):
        if cls._instance is None:
            cls._instance = super().__call__()
        return cls._instance


class Config(metaclass=ConfigMeta):
    """Class that contains the application settings"""

    def __init__(self, config_file=CONFIG_FILE_PATH):
        self.config_file = os.path.expanduser(config_file)
        self.data = self._load_config()

    def _load_config(self):
        """Load configuration from toml file"""
        if not os.path.exists(self.config_file):
            return {"registry": {}}

        try:
            with open(self.config_file) as f:
                return toml.load(f)

        except toml.TomlDecodeError:
            logger.error(
                f"Configuration file does not respect toml format:"
                f"{self.config_file}"
            )
            sys.exit(1)

    def reset(self):
        """Reset the configuration"""
        self.data = self._load_config()

    def save(self):
        """Save the config"""
        with open(self.config_file, "w") as f:
            toml.dump(self.data, f)



import os

import click

URL_TEMPLATE = "\e]8;;{url}\a{title}\e]8;;\a"


def info(msg):
    click.echo(click.wrap_text(msg))


def error(msg):
    click.echo(click.style(msg, fg="red"))


def paragraph(msg):
    click.echo(click.wrap_text(msg, initial_indent=" ", subsequent_indent=" "))


def link(url, title):
    os.system(
        f"""echo -e '{URL_TEMPLATE.format(url=url, title=click.wrap_text(title))}'"""
    )



import click

from myprss.reader import read_feed
from myprss.registry import registry

# echo -e '\e]8;;http://example.com\aThis is a link\e]8;;\a'


@click.group(options_metavar="")
def cli():
    pass


cli.add_command(registry)
cli.add_command(read_feed)


if __name__ == "__main__":
    cli()



from defusedxml.ElementTree import XML

from myprss import logger


def parse_rss_content(data):
    rss = XML(data)

    if rss.get("version") != "2.0":
        logger.error("Only rss version 2.0 are supported for now")
        return

    channel = rss.find("channel")

    data = {}

    data["site"] = channel.find("title").text
    data["items"] = []

    for item in channel.findall("item"):
        data["items"].append(
            {
                "date": item.find("pubDate").text,
                "title": item.find("title").text,
                "url": item.find("link").text,
                "description": item.find("description").text,
            }
        )

    return data



import click

from myprss import logger
from myprss.config import Config
from myprss.utils import list_known_feeds


@click.group("registry", help="Manage your list of rss feeds")
def registry():
    pass


@registry.command(
    "add",
    help="Add a rss feed to the registry",
    no_args_is_help=True,
    options_metavar="",
)
@click.argument("name", required=True)
@click.argument("url", required=True)
def register_new_feed(name, url):
    config = Config()
    config.data["registry"][name] = url
    config.save()
    logger.info(f"{name} has been added to the rss registry")


@registry.command("list", help="List all saved rss feeds")
def list_all_feed():
    config = Config()

    if not config.data["registry"]:
        logger.info("The registry is empty")
        return

    max_name_size = max(len(name) for name in config.data["registry"].keys())

    titles = f"{'NAME':{max_name_size}}\t| URL"

    logger.info(titles)
    for name, url in config.data["registry"].items():
        logger.info(f"{name:{max_name_size}}\t| {url}")


@registry.command(
    "delete",
    help="Delete a rss feed from the registry",
    no_args_is_help=True,
    options_metavar="",
)
@click.argument("name", required=True, autocompletion=list_known_feeds)
def delete_feed(name):
    config = Config()
    if name not in config.data["registry"]:
        logger.error(f"{name} is unknown")
        return

    del config.data["registry"][name]
    config.save()
    logger.info(f"{name} has been deleted from the rss registry")



import click
import requests

from myprss import logger
from myprss.config import Config
from myprss.parser import parse_rss_content
from myprss.utils import list_known_feeds


@click.command(
    "read", help="Read a rss feed", no_args_is_help=True, options_metavar=""
)
@click.argument("name", autocompletion=list_known_feeds)
def read_feed(name):
    config = Config()

    if name not in config.data["registry"]:
        logger.error(f"Unknown rss feed: {name}")
        return

    url = config.data["registry"][name]
    response = requests.get(url)

    if response.status_code != 200:
        logger.error(f"{response.reason} -> {url}")
        return

    content = parse_rss_content(response.content)

    logger.info(content["site"])

    for item in reversed(content["items"]):
        logger.info(item["date"])
        logger.link(item["url"], item["title"])
        logger.paragraph(item["description"])
        logger.info("")
